<?php

namespace App\Git;

use GitWrapper\GitWrapper;
use Illuminate\Support\Facades\Storage;

class CloneTask
{
    private const MAX_TIMEOUT = 99999;

    /**
     * @var \App\Git\Cloneable
     */
    private $cloneable;

    public function __construct(Cloneable $cloneable)
    {
        $this->cloneable = $cloneable;
    }

    public function __invoke()
    {
        if (Storage::disk('git-clone')->exists($this->cloneable->getFolder())) {
            $git = new GitWrapper;
            $workingCopy = $git->workingCopy(Storage::disk('git-clone')->path($this->cloneable->getFolder()));
            $workingCopy->pull();

            // TODO: compare remote urls and make sure they are the same

            return true;
        }

        $git = new GitWrapper;
        $git->setTimeout(self::MAX_TIMEOUT);

        return $git->cloneRepository(
            $this->cloneable->getCloneUrl(),
            Storage::disk('git-clone')->path($this->cloneable->getFolder())
        );
    }
}
