<?php

namespace App\Git;

use Gitlab\Client as GitlabClient;
use Gitlab\ResultPager;
use LaravelZero\Framework\Commands\Command;

class ProjectPicker
{
    /**
     * @var Command
     */
    private $output;

    public function __construct(Command $output)
    {
        $this->output = $output;
    }

    public function pick(string $accessToken)
    {
        $projects = [];
        $this->output->task(
            'Fetching all projects. This may take a few minutes',
            static function () use ($accessToken, &$projects) {
                $client = GitlabClient::create('https://git.drp.digital')
                    ->authenticate($accessToken, GitlabClient::AUTH_OAUTH_TOKEN);
                $pager = new ResultPager($client);

                $projects = $pager->fetchAll($client->api('projects'), 'all', [['simple' => true]]);
            }
        );

        $namespacedProjects = collect($projects)->groupBy('namespace.name')->sortBy('namespace.name');
        $namespace = $this->output->choice('What namespace', $namespacedProjects->keys()->toArray());
        $namespaceProjects = $namespacedProjects->get($namespace)->keyBy('name');
        $projectName = $this->output->choice('What project?', $namespaceProjects->pluck('name')->toArray());
        $pickedProject = $namespaceProjects->get($projectName);

        return $pickedProject['path_with_namespace'];
    }
}
