<?php

namespace App\Installation;

use App\Database\Env\Writer;
use App\Installation\Commands\ComposerInstall;
use App\Installation\Commands\CreateDatabase;
use App\Installation\Contracts\Instructions;
use App\Installation\Contracts\RequiresDatabase;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class DrupalInstructions extends Instructions implements RequiresDatabase
{
    public function name(): string
    {
        return 'Drupal';
    }

    public function canRun(): bool
    {
        if (Storage::disk('project')->exists('composer.json') === false) {
            return false;
        }

        $composerJson = Storage::disk('project')->get('composer.json');

        return Str::contains($composerJson, 'drupal/core');
    }

    protected function commands(): array
    {
        return array_filter([
            $this->command('cp .env.example .env'),
            new CreateDatabase(new Writer([
                'dbHost' => 'MYSQL_HOSTNAME',
                'dbPort' => 'MYSQL_PORT',
                'dbName' => 'MYSQL_DATABASE',
                'dbUsername' => 'MYSQL_USER',
                'dbPassword' => 'MYSQL_PASSWORD',
            ])),
            new ComposerInstall,
        ]);
    }
}
