<?php

namespace App\Installation;

use App\Database\Env\Writer;
use App\Installation\Commands\Artisan;
use App\Installation\Commands\ComposerInstall;
use App\Installation\Commands\Contracts\Command;
use App\Installation\Commands\CreateDatabase;
use App\Installation\Commands\NpmBuild;
use App\Installation\Commands\NpmInstall;
use App\Installation\Contracts\Instructions;
use App\Installation\Contracts\RequiresDatabase;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Storage;

class LaravelInstructions extends Instructions implements RequiresDatabase
{
    public function name(): string
    {
        return 'Laravel';
    }

    public function canRun(): bool
    {
        return Storage::disk('project')->exists('artisan');
    }

    protected function commands(): array
    {
        return array_filter([
            $this->command('cp .env.example .env'),
            new ComposerInstall,
            $this->artisan('key:generate'),
            // new CreateDatabase(new Writer([
            //     'dbHost' => 'DB_HOST',
            //     'dbPort' => 'DB_PORT',
            //     'dbUsername' => 'DB_USERNAME',
            //     'dbPassword' => 'DB_PASSWORD',
            //     'dbName' => 'DB_DATABASE',
            // ])),
            $this->artisan('migrate')->options(['--seed']),
            function () {
                return $this->setup();
            },
            // new NpmInstall,
            // new NpmBuild,
        ]);
    }

    private function setup(): ?Command
    {
        $artisan = $this->artisan('list')->options(['--format=json']);
        $artisan->silentExecute();
        $listCommands = json_decode($artisan->getOutput(), true);

        if ($listCommands === null) {
            return null;
        }

        $listCommands = array_map(static function ($command): ?string {
            return Arr::get($command, 'name');
        }, $listCommands['commands']);

        if (in_array('setup', $listCommands, true)) {
            return $this->artisan('setup');
        }

        if (in_array('app:install', $listCommands, true)) {
            return $this->artisan('app:install');
        }

        return null;
    }

    private function artisan(string $command): Artisan
    {
        return new Artisan($command);
    }
}
